# -*- coding: utf-8 -*-
"""
rsCase is a simple python package implemented in Rust for string case translation.
"""


__author__ = "Sondre Lillebø Gundersen <sondrelg@live.no>"
__version__ = "1.0.1"
__all__ = ["rscase"]
